@@ -26,6 +26,7 @@
 //! ));
 //! ```
 
+use std::collections::HashMap;
 use twilight_model::{
     application::command::{
         BaseCommandOptionData, ChannelCommandOptionData, ChoiceCommandOptionData, Command,
@@ -33,8 +34,10 @@ use twilight_model::{
         NumberCommandOptionData, OptionsCommandOptionData,
     },
     channel::ChannelType,
+    guild::Permissions,
     id::{marker::GuildMarker, Id},
 };
+use twilight_validate::command::{command as validate_command, CommandValidationError};
 
 /// Builder to create a [`Command`].
 #[allow(clippy::module_name_repetitions)]
@@ -48,12 +51,15 @@ impl CommandBuilder {
     pub const fn new(name: String, description: String, kind: CommandType) -> Self {
         Self(Command {
             application_id: None,
-            default_permission: None,
+            default_member_permissions: None,
+            dm_permission: None,
             description,
+            description_localizations: None,
             guild_id: None,
             id: None,
             kind,
             name,
+            name_localizations: None,
             options: Vec::new(),
             version: Id::new(1),
         })
@@ -66,6 +72,18 @@ impl CommandBuilder {
         self.0
     }
 
+    /// Ensure the command is valid.
+    ///
+    /// # Errors
+    ///
+    /// Refer to the errors section of [`twilight_validate::command::command`]
+    /// for possible errors.
+    pub fn validate(self) -> Result<Self, CommandValidationError> {
+        validate_command(&self.0)?;
+
+        Ok(self)
+    }
+
     /// Set the guild ID of the command.
     ///
     /// Defaults to [`None`].
@@ -75,11 +93,43 @@ impl CommandBuilder {
         self
     }
 
-    /// Set the default permission of the command.
+    /// Set the default member permission required to run the command.
     ///
     /// Defaults to [`None`].
-    pub const fn default_permission(mut self, default_permission: bool) -> Self {
-        self.0.default_permission = Some(default_permission);
+    pub const fn default_member_permissions(
+        mut self,
+        default_member_permissions: Permissions,
+    ) -> Self {
+        self.0.default_member_permissions = Some(default_member_permissions);
+
+        self
+    }
+
+    /// Set whether the command is available in DMs.
+    ///
+    /// Defaults to [`None`].
+    pub const fn dm_permission(mut self, dm_permission: bool) -> Self {
+        self.0.dm_permission = Some(dm_permission);
+
+        self
+    }
+
+    /// Set the localization dictionary for the command description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the command name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
 
         self
     }
@@ -109,7 +159,9 @@ impl AttachmentBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(BaseCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -121,6 +173,26 @@ impl AttachmentBuilder {
         CommandOption::Attachment(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -148,7 +220,9 @@ impl BooleanBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(BaseCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -160,6 +234,26 @@ impl BooleanBuilder {
         CommandOption::Boolean(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -188,7 +282,9 @@ impl ChannelBuilder {
         Self(ChannelCommandOptionData {
             channel_types: Vec::new(),
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -209,6 +305,26 @@ impl ChannelBuilder {
         self
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -237,9 +353,11 @@ impl IntegerBuilder {
             autocomplete: false,
             choices: Vec::new(),
             description,
+            description_localizations: None,
             max_value: None,
             min_value: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -260,20 +378,59 @@ impl IntegerBuilder {
         self
     }
 
+    /// Set localization for a particular choice, by name.
+    ///
+    /// Choices must be set with the [`choices`] method before updating their
+    /// localization.
+    ///
+    /// [`choices`]: Self::choices
+    pub fn choice_localizations(
+        mut self,
+        choice_name: &str,
+        name_localizations: HashMap<String, String>,
+    ) -> Self {
+        let choice = self.0.choices.iter_mut().find(
+            |choice| matches!(choice, CommandOptionChoice::Int { name, .. } if name == choice_name),
+        );
+
+        if let Some(choice) = choice {
+            set_choice_localizations(choice, name_localizations);
+        }
+
+        self
+    }
+
     /// Set the list of choices for an option.
     ///
     /// Accepts tuples of `(String, i64)` corresponding to the name and value.
+    /// Localization may be added with [`choice_localizations`].
     ///
     /// Defaults to no choices.
+    ///
+    /// [`choice_localizations`]: Self::choice_localizations
     pub fn choices(mut self, choices: impl IntoIterator<Item = (String, i64)>) -> Self {
         self.0.choices = choices
             .into_iter()
-            .map(|(name, value)| CommandOptionChoice::Int { name, value })
+            .map(|(name, value, ..)| CommandOptionChoice::Int {
+                name,
+                name_localizations: None,
+                value,
+            })
             .collect();
 
         self
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set the maximum value permitted.
     ///
     /// Defaults to no limit.
@@ -292,6 +449,16 @@ impl IntegerBuilder {
         self
     }
 
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -319,7 +486,9 @@ impl MentionableBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(BaseCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -331,6 +500,26 @@ impl MentionableBuilder {
         CommandOption::Mentionable(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -360,9 +549,11 @@ impl NumberBuilder {
             autocomplete: false,
             choices: Vec::new(),
             description,
+            description_localizations: None,
             max_value: None,
             min_value: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -383,21 +574,59 @@ impl NumberBuilder {
         self
     }
 
+    /// Set localization for a particular choice, by name.
+    ///
+    /// Choices must be set with the [`choices`] method before updating their
+    /// localization.
+    ///
+    /// [`choices`]: Self::choices
+    pub fn choice_localizations(
+        mut self,
+        choice_name: &str,
+        name_localizations: HashMap<String, String>,
+    ) -> Self {
+        let choice = self.0.choices.iter_mut().find(
+            |choice| matches!(choice, CommandOptionChoice::Number { name, .. } if name == choice_name),
+        );
+
+        if let Some(choice) = choice {
+            set_choice_localizations(choice, name_localizations);
+        }
+
+        self
+    }
+
     /// Set the list of choices for an option.
     ///
     /// Accepts tuples of `(String, Number)` corresponding to the name and
-    /// value.
+    /// value. Localization may be added with [`choice_localizations`].
     ///
     /// Defaults to no choices.
+    ///
+    /// [`choice_localizations`]: Self::choice_localizations
     pub fn choices(mut self, choices: impl IntoIterator<Item = (String, Number)>) -> Self {
         self.0.choices = choices
             .into_iter()
-            .map(|(name, value)| CommandOptionChoice::Number { name, value })
+            .map(|(name, value, ..)| CommandOptionChoice::Number {
+                name,
+                name_localizations: None,
+                value,
+            })
             .collect();
 
         self
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set the maximum value permitted.
     ///
     /// Defaults to no limit.
@@ -416,6 +645,16 @@ impl NumberBuilder {
         self
     }
 
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -443,7 +682,9 @@ impl RoleBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(BaseCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -455,6 +696,26 @@ impl RoleBuilder {
         CommandOption::Role(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -484,7 +745,9 @@ impl StringBuilder {
             autocomplete: false,
             choices: Vec::new(),
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -505,21 +768,69 @@ impl StringBuilder {
         self
     }
 
+    /// Set localization for a particular choice, by name.
+    ///
+    /// Choices must be set with the [`choices`] method before updating their
+    /// localization.
+    ///
+    /// [`choices`]: Self::choices
+    pub fn choice_localizations(
+        mut self,
+        choice_name: &str,
+        name_localizations: HashMap<String, String>,
+    ) -> Self {
+        let choice = self.0.choices.iter_mut().find(
+            |choice| matches!(choice, CommandOptionChoice::String { name, .. } if name == choice_name),
+        );
+
+        if let Some(choice) = choice {
+            set_choice_localizations(choice, name_localizations);
+        }
+
+        self
+    }
+
     /// Set the list of choices for an option.
     ///
     /// Accepts tuples of `(String, String)` corresponding to the name and
-    /// value.
+    /// value. Localization may be added with [`choice_localizations`].
     ///
     /// Defaults to no choices.
+    ///
+    /// [`choice_localizations`]: Self::choice_localizations
     pub fn choices(mut self, choices: impl IntoIterator<Item = (String, String)>) -> Self {
         self.0.choices = choices
             .into_iter()
-            .map(|(name, value)| CommandOptionChoice::String { name, value })
+            .map(|(name, value, ..)| CommandOptionChoice::String {
+                name,
+                name_localizations: None,
+                value,
+            })
             .collect();
 
         self
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -547,7 +858,9 @@ impl SubCommandBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(OptionsCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             options: Vec::new(),
         })
     }
@@ -559,6 +872,26 @@ impl SubCommandBuilder {
         CommandOption::SubCommand(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Add an option to the sub command.
     ///
     /// Defaults to an empty list.
@@ -590,7 +923,9 @@ impl SubCommandGroupBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(OptionsCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             options: Vec::new(),
         })
     }
@@ -602,6 +937,26 @@ impl SubCommandGroupBuilder {
         CommandOption::SubCommandGroup(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set the list of sub commands to the group.
     ///
     /// Defaults to no subcommands.
@@ -629,7 +984,9 @@ impl UserBuilder {
     pub const fn new(name: String, description: String) -> Self {
         Self(BaseCommandOptionData {
             description,
+            description_localizations: None,
             name,
+            name_localizations: None,
             required: false,
         })
     }
@@ -641,6 +998,26 @@ impl UserBuilder {
         CommandOption::User(self.0)
     }
 
+    /// Set the localization dictionary for the option description.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn description_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.description_localizations = Some(localizations);
+
+        self
+    }
+
+    /// Set the localization dictionary for the option name.
+    ///
+    /// Defaults to [`None`].
+    #[allow(clippy::missing_const_for_fn)]
+    pub fn name_localizations(mut self, localizations: HashMap<String, String>) -> Self {
+        self.0.name_localizations = Some(localizations);
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -657,6 +1034,25 @@ impl From<UserBuilder> for CommandOption {
     }
 }
 
+fn set_choice_localizations(
+    choice: &mut CommandOptionChoice,
+    localizations: HashMap<String, String>,
+) {
+    let name_localizations = match choice {
+        CommandOptionChoice::String {
+            name_localizations, ..
+        }
+        | CommandOptionChoice::Int {
+            name_localizations, ..
+        }
+        | CommandOptionChoice::Number {
+            name_localizations, ..
+        } => name_localizations,
+    };
+
+    *name_localizations = Some(localizations);
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -748,24 +1144,33 @@ mod tests {
 
         let command_manual = Command {
             application_id: None,
+            default_member_permissions: None,
+            dm_permission: None,
+            description: String::from("Get or edit permissions for a user or a role"),
             guild_id: None,
+            id: None,
             kind: CommandType::ChatInput,
             name: String::from("permissions"),
-            default_permission: None,
-            description: String::from("Get or edit permissions for a user or a role"),
-            id: None,
+            name_localizations: None,
+            description_localizations: None,
             options: Vec::from([
                 CommandOption::SubCommandGroup(OptionsCommandOptionData {
                     description: String::from("Get or edit permissions for a user"),
+                    description_localizations: None,
                     name: String::from("user"),
+                    name_localizations: None,
                     options: Vec::from([
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Get permissions for a user"),
+                            description_localizations: None,
                             name: String::from("get"),
+                            name_localizations: None,
                             options: Vec::from([
                                 CommandOption::User(BaseCommandOptionData {
                                     description: String::from("The user to get"),
+                                    description_localizations: None,
                                     name: String::from("user"),
+                                    name_localizations: None,
                                     required: true,
                                 }),
                                 CommandOption::Channel(ChannelCommandOptionData {
@@ -774,18 +1179,24 @@ mod tests {
                                         "The channel permissions to get. If omitted, the guild \
                                          permissions will be returned",
                                     ),
+                                    description_localizations: None,
                                     name: String::from("channel"),
+                                    name_localizations: None,
                                     required: false,
                                 }),
                             ]),
                         }),
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Edit permissions for a user"),
+                            description_localizations: None,
                             name: String::from("edit"),
+                            name_localizations: None,
                             options: Vec::from([
                                 CommandOption::User(BaseCommandOptionData {
                                     description: String::from("The user to edit"),
+                                    description_localizations: None,
                                     name: String::from("user"),
+                                    name_localizations: None,
                                     required: true,
                                 }),
                                 CommandOption::Channel(ChannelCommandOptionData {
@@ -794,7 +1205,9 @@ mod tests {
                                         "The channel permissions to edit. If omitted, the guild \
                                          permissions will be edited",
                                     ),
+                                    description_localizations: None,
                                     name: String::from("channel"),
+                                    name_localizations: None,
                                     required: false,
                                 }),
                             ]),
@@ -803,15 +1216,21 @@ mod tests {
                 }),
                 CommandOption::SubCommandGroup(OptionsCommandOptionData {
                     description: String::from("Get or edit permissions for a role"),
+                    description_localizations: None,
                     name: String::from("role"),
+                    name_localizations: None,
                     options: Vec::from([
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Get permissions for a role"),
+                            description_localizations: None,
                             name: String::from("get"),
+                            name_localizations: None,
                             options: Vec::from([
                                 CommandOption::Role(BaseCommandOptionData {
                                     description: String::from("The role to get"),
+                                    description_localizations: None,
                                     name: String::from("role"),
+                                    name_localizations: None,
                                     required: true,
                                 }),
                                 CommandOption::Channel(ChannelCommandOptionData {
@@ -820,18 +1239,24 @@ mod tests {
                                         "The channel permissions to get. If omitted, the guild \
                                          permissions will be returned",
                                     ),
+                                    description_localizations: None,
                                     name: String::from("channel"),
+                                    name_localizations: None,
                                     required: false,
                                 }),
                             ]),
                         }),
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Edit permissions for a role"),
+                            description_localizations: None,
                             name: String::from("edit"),
+                            name_localizations: None,
                             options: Vec::from([
                                 CommandOption::Role(BaseCommandOptionData {
                                     description: String::from("The role to edit"),
+                                    description_localizations: None,
                                     name: String::from("role"),
+                                    name_localizations: None,
                                     required: true,
                                 }),
                                 CommandOption::Channel(ChannelCommandOptionData {
@@ -840,16 +1265,20 @@ mod tests {
                                         "The channel permissions to edit. If omitted, the guild \
                                          permissions will be edited",
                                     ),
+                                    description_localizations: None,
                                     name: String::from("channel"),
+                                    name_localizations: None,
                                     required: false,
                                 }),
                                 CommandOption::Number(NumberCommandOptionData {
                                     autocomplete: true,
                                     choices: Vec::new(),
                                     description: String::from("The position of the new role"),
+                                    description_localizations: None,
                                     max_value: None,
                                     min_value: None,
                                     name: String::from("position"),
+                                    name_localizations: None,
                                     required: false,
                                 }),
                             ]),
@@ -862,4 +1291,11 @@ mod tests {
 
         assert_eq!(command, command_manual);
     }
+
+    #[test]
+    fn test_validate() {
+        let result = CommandBuilder::new("".into(), "".into(), CommandType::ChatInput).validate();
+
+        assert!(result.is_err());
+    }
 }
